# Generated by Django 5.2.4 on 2025-07-15 16:37

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0002_modeld_modela_model_d'),
    ]

    operations = [
        migrations.AddField(
            model_name='modelc',
            name='boolean',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='modela',
            name='model_b',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='example.modelb'),
        ),
        migrations.AlterField(
            model_name='modelb',
            name='chars',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='modelb',
            name='date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='modelb',
            name='datetime',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='modelb',
            name='decimal',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True),
        ),
        migrations.AlterField(
            model_name='modelb',
            name='integer',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
