# Generated by Django 5.1.3 on 2025-07-10 10:11

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ModelB',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('chars', models.CharField(max_length=255)),
                ('integer', models.IntegerField()),
                ('decimal', models.DecimalField(decimal_places=2, max_digits=5)),
                ('date', models.DateField()),
                ('datetime', models.DateTimeField()),
            ],
        ),
        migrations.CreateModel(
            name='ModelC',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('chars', models.CharField(max_length=255)),
                ('integer', models.IntegerField()),
                ('date', models.DateField()),
            ],
        ),
        migrations.CreateModel(
            name='ModelA',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('boolean', models.BooleanField(null=True)),
                ('chars', models.CharField(max_length=255)),
                ('chars_choices', models.CharField(choices=[('one', 'One'), ('two', 'Two'), ('three', 'Three'), ('four', 'Four')], max_length=255)),
                ('text', models.TextField()),
                ('email', models.EmailField(max_length=254)),
                ('url', models.URLField()),
                ('uuid', models.UUIDField()),
                ('integer', models.IntegerField()),
                ('big_integer', models.BigIntegerField()),
                ('integer_choices', models.IntegerField(choices=[(1, 1), (2, 2), (3, 3), (4, 4)])),
                ('float', models.FloatField()),
                ('decimal', models.DecimalField(decimal_places=2, max_digits=5)),
                ('date', models.DateField()),
                ('time', models.TimeField()),
                ('datetime', models.DateTimeField()),
                ('model_b', models.OneToOneField(null=True, on_delete=django.db.models.deletion.CASCADE, to='example.modelb')),
            ],
        ),
        migrations.AddField(
            model_name='modelb',
            name='model_c',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='example.modelc'),
        ),
    ]
