# Generated by Django 5.1.3 on 2025-05-22 19:04

import django.db.models.deletion
import picklefield.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='SearchkitSearch',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Search name')),
                ('data', picklefield.fields.PickledObjectField(editable=False, verbose_name='Serialized filter rule data')),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('contenttype', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype', verbose_name='Model')),
            ],
            options={
                'unique_together': {('name', 'contenttype')},
            },
        ),
    ]
