# Generated by Django 5.1.3 on 2025-06-07 05:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('searchkit', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='SearchkitSearch',
            new_name='Search',
        ),
    ]
